export interface IPlayerJoin {
    readonly addr: string;
    readonly position: number;
    readonly balance: bigint;
    readonly accessVersion: bigint;
    readonly verifyKey: string;
}
export interface IPlayerDeposit {
    readonly addr: string;
    readonly amount: bigint;
    readonly settleVersion: bigint;
}
export interface IServerJoin {
    readonly addr: string;
    readonly endpoint: string;
    readonly accessVersion: bigint;
    readonly verifyKey: string;
}
export declare enum VoteType {
    ServerVoteTransactorDropOff = 0,
    ClientVoteTransactorDropOff = 1
}
export interface IVote {
    readonly voter: string;
    readonly votee: string;
    readonly voteType: VoteType;
}
export interface IGameRegistration {
    readonly title: string;
    readonly addr: string;
    readonly regTime: bigint;
    readonly bundleAddr: string;
}
export interface IGameAccount {
    readonly addr: string;
    readonly title: string;
    readonly bundleAddr: string;
    readonly tokenAddr: string;
    readonly ownerAddr: string;
    readonly settleVersion: bigint;
    readonly accessVersion: bigint;
    readonly players: PlayerJoin[];
    readonly deposits: PlayerDeposit[];
    readonly servers: ServerJoin[];
    readonly transactorAddr: string | undefined;
    readonly votes: Vote[];
    readonly unlockTime: bigint | undefined;
    readonly maxPlayers: number;
    readonly dataLen: number;
    readonly data: Uint8Array;
    readonly minDeposit: bigint;
    readonly maxDeposit: bigint;
}
export interface IServerAccount {
    readonly addr: string;
    readonly endpoint: string;
}
export interface IGameBundle {
    readonly uri: string;
    readonly name: string;
    readonly data: Uint8Array;
}
export interface IPlayerProfile {
    readonly addr: string;
    readonly nick: string;
    readonly pfp: string | undefined;
}
export interface IRegistrationAccount {
    readonly addr: string;
    readonly isPrivate: boolean;
    readonly size: number;
    readonly owner: string | undefined;
    readonly games: GameRegistration[];
}
export interface IToken {
    readonly addr: string;
    readonly icon: string;
    readonly name: string;
    readonly symbol: string;
    readonly decimals: number;
}
export declare class Token implements IToken {
    readonly addr: string;
    readonly icon: string;
    readonly name: string;
    readonly symbol: string;
    readonly decimals: number;
    constructor(fields: IToken);
}
export declare class TokenWithBalance implements IToken {
    readonly addr: string;
    readonly icon: string;
    readonly name: string;
    readonly symbol: string;
    readonly decimals: number;
    readonly amount: bigint;
    readonly uiAmount: string;
    constructor(token: IToken, amount: bigint);
}
export interface INft {
    readonly addr: string;
    readonly image: string;
    readonly name: string;
    readonly symbol: string;
    readonly collection: string | undefined;
}
export declare class ServerAccount implements IServerAccount {
    readonly addr: string;
    readonly endpoint: string;
    constructor(fields: IServerAccount);
}
export declare class PlayerJoin implements IPlayerJoin {
    readonly addr: string;
    readonly position: number;
    readonly balance: bigint;
    readonly accessVersion: bigint;
    readonly verifyKey: string;
    constructor(fields: IPlayerJoin);
}
export declare class ServerJoin implements IServerJoin {
    readonly addr: string;
    readonly endpoint: string;
    readonly accessVersion: bigint;
    readonly verifyKey: string;
    constructor(fields: IServerJoin);
}
export declare class PlayerDeposit implements IPlayerDeposit {
    readonly addr: string;
    readonly amount: bigint;
    readonly settleVersion: bigint;
    constructor(fields: IPlayerDeposit);
}
export declare class Vote implements IVote {
    readonly voter: string;
    readonly votee: string;
    readonly voteType: VoteType;
    constructor(fields: IVote);
}
export declare class GameAccount implements IGameAccount {
    readonly addr: string;
    readonly title: string;
    readonly bundleAddr: string;
    readonly tokenAddr: string;
    readonly ownerAddr: string;
    readonly settleVersion: bigint;
    readonly accessVersion: bigint;
    readonly players: PlayerJoin[];
    readonly deposits: PlayerDeposit[];
    readonly servers: ServerJoin[];
    readonly transactorAddr: string | undefined;
    readonly votes: Vote[];
    readonly unlockTime: bigint | undefined;
    readonly maxPlayers: number;
    readonly minDeposit: bigint;
    readonly maxDeposit: bigint;
    readonly dataLen: number;
    readonly data: Uint8Array;
    constructor(fields: IGameAccount);
}
export declare class GameBundle implements IGameBundle {
    readonly uri: string;
    readonly name: string;
    readonly data: Uint8Array;
    constructor(fields: IGameBundle);
}
export declare class GameRegistration implements IGameRegistration {
    readonly title: string;
    readonly addr: string;
    readonly regTime: bigint;
    readonly bundleAddr: string;
    constructor(fields: IGameRegistration);
}
export declare class RegistrationAccount implements IRegistrationAccount {
    readonly addr: string;
    readonly isPrivate: boolean;
    readonly size: number;
    readonly owner: string | undefined;
    readonly games: GameRegistration[];
    constructor(fields: IRegistrationAccount);
}
/**
 * The registration account data with games consolidated.
 */
export declare class RegistrationWithGames {
    readonly addr: string;
    readonly isPrivate: boolean;
    readonly size: number;
    readonly owner: string | undefined;
    readonly games: GameAccount[];
    constructor(fields: Object);
}
export declare class PlayerProfile implements IPlayerProfile {
    readonly addr: string;
    readonly nick: string;
    readonly pfp: string | undefined;
    constructor(fields: IPlayerProfile);
}
