import { IConnection } from './connection';
import { GameContext } from './game-context';
import { GameContextSnapshot } from './game-context-snapshot';
import { ITransport } from './transport';
import { IWallet } from './wallet';
import { Handler } from './handler';
import { IEncryptor } from './encryptor';
import { GameAccount, PlayerProfile } from './accounts';
import { Client } from './client';
import { GameEvent, ICustomEvent } from './events';
export type EventCallbackFunction = (context: GameContextSnapshot, state: Uint8Array, event: GameEvent | undefined) => void;
export type AppClientInitOpts = {
    transport: ITransport;
    wallet: IWallet;
    gameAddr: string;
    callback: EventCallbackFunction;
};
export type JoinOpts = {
    amount: bigint;
    position?: number;
};
export declare class AppClient {
    #private;
    constructor(gameAddr: string, handler: Handler, wallet: IWallet, client: Client, transport: ITransport, connection: IConnection, gameContext: GameContext, initGameAccount: GameAccount, callback: EventCallbackFunction, encryptor: IEncryptor);
    static initialize(opts: AppClientInitOpts): Promise<AppClient>;
    get playerAddr(): string;
    get gameAddr(): string;
    get gameContext(): GameContext;
    /**
     * Get player profile by its wallet address.
     */
    getProfile(addr: string): Promise<PlayerProfile | undefined>;
    invokeCallback(event: GameEvent | undefined): Promise<void>;
    /**
     * Connect to the transactor and retrieve the event stream.
     */
    attachGame(): Promise<void>;
    /**
     * Join game.
     */
    join(params: JoinOpts): Promise<void>;
    /**
     * Submit an event.
     */
    submitEvent(raw: Uint8Array): Promise<void>;
    submitEvent(customEvent: ICustomEvent): Promise<void>;
    /**
     * Get hidden knowledge by random id. The result contains both
     * public and private information.  For performance reason, it's
     * better to cache the result somewhere instead of calling this
     * function frequently.
     */
    getRevealed(randomId: number): Promise<Map<number, string>>;
    /**
     * Close current event subscription.
     */
    close(): Promise<void>;
    /**
     * Exit current game.
     */
    exit(): Promise<void>;
}
