import { IEncryptor, PublicKeyRaws } from './encryptor';
import { GameEvent } from './events';
type Method = 'attach_game' | 'submit_event' | 'exit_game' | 'subscribe_event';
interface IAttachGameParams {
    signer: string;
    key: PublicKeyRaws;
}
interface ISubscribeEventParams {
    settleVersion: bigint;
}
interface ISubmitEventParams {
    event: GameEvent;
}
export declare class AttachGameParams {
    signer: string;
    key: PublicKeyRaws;
    constructor(fields: IAttachGameParams);
}
export declare class ExitGameParams {
}
export declare class SubscribeEventParams {
    settleVersion: bigint;
    constructor(fields: ISubscribeEventParams);
}
export declare class SubmitEventParams {
    event: GameEvent;
    constructor(fields: ISubmitEventParams);
}
export declare abstract class BroadcastFrame {
}
export declare class BroadcastFrameEvent extends BroadcastFrame {
    gameAddr: string;
    event: GameEvent;
    timestamp: bigint;
    constructor(fields: any);
}
export declare class BroadcastFrameInit extends BroadcastFrame {
    gameAddr: string;
    accessVersion: bigint;
    settleVersion: bigint;
    constructor(fields: any);
}
export interface BroadcastFrame {
    gameAddr: string;
    state: any;
    event: GameEvent;
}
export interface IConnection {
    attachGame(gameAddr: string, params: AttachGameParams): Promise<void>;
    submitEvent(gameAddr: string, params: SubmitEventParams): Promise<void>;
    exitGame(gameAddr: string, params: ExitGameParams): Promise<void>;
    subscribeEvents(gameAddr: string, params: SubscribeEventParams): AsyncGenerator<BroadcastFrame | undefined>;
}
export declare class Connection implements IConnection {
    #private;
    constructor(playerAddr: string, endpoint: string, encryptor: IEncryptor);
    attachGame(gameAddr: string, params: AttachGameParams): Promise<void>;
    submitEvent(gameAddr: string, params: SubmitEventParams): Promise<void>;
    exitGame(gameAddr: string, params: ExitGameParams): Promise<void>;
    subscribeEvents(gameAddr: string, params: SubscribeEventParams): AsyncGenerator<BroadcastFrame | undefined>;
    parseEventMessage(raw: string): BroadcastFrame | undefined;
    static initialize(playerAddr: string, endpoint: string, encryptor: IEncryptor): Connection;
    makeReq<P>(gameAddr: string, method: Method, params: P): Promise<string>;
    makeReqNoSig<P>(gameAddr: string, method: Method, params: P): string;
    requestWs(req: string): Promise<void>;
    requestXhr<P>(req: string): Promise<P>;
    waitSocketReady(): Promise<unknown>;
}
export {};
