import { PlayerJoin, ServerJoin } from './accounts';
import { Fields, Id } from './types';
export interface ICustomEvent {
    serialize(): Uint8Array;
}
export declare abstract class SecretShare {
}
export declare class Random extends SecretShare {
    fromAddr: string;
    toAddr: string | undefined;
    randomId: Id;
    index: number;
    secret: Uint8Array;
    constructor(fields: Fields<Random>);
}
export declare class Answer extends SecretShare {
    fromAddr: string;
    decisionId: Id;
    secret: Uint8Array;
    constructor(fields: Fields<Answer>);
}
export declare abstract class GameEvent {
}
export declare class Custom extends GameEvent {
    sender: string;
    raw: Uint8Array;
    constructor(fields: Fields<Custom>);
}
export declare function makeCustomEvent(sender: string, customEvent: ICustomEvent): Custom;
export declare class Ready extends GameEvent {
    constructor(_?: any);
}
export declare class ShareSecrets extends GameEvent {
    sender: string;
    shares: SecretShare[];
    constructor(fields: Fields<ShareSecrets>);
}
export declare class OperationTimeout extends GameEvent {
    addrs: string[];
    constructor(fields: Fields<OperationTimeout>);
}
export declare class Mask extends GameEvent {
    sender: string;
    randomId: Id;
    ciphertexts: Uint8Array[];
    constructor(fields: Fields<Mask>);
}
export declare class CiphertextAndDigest {
    ciphertext: Uint8Array;
    digest: Uint8Array;
    constructor(fields: Fields<CiphertextAndDigest>);
}
export declare class Lock extends GameEvent {
    sender: string;
    randomId: Id;
    ciphertextsAndDigests: CiphertextAndDigest[];
    constructor(fields: Fields<Lock>);
}
export declare class RandomnessReady extends GameEvent {
    randomId: Id;
    constructor(fields: Fields<RandomnessReady>);
}
export declare class Sync extends GameEvent {
    newPlayers: PlayerJoin[];
    newServers: ServerJoin[];
    transactorAddr: string;
    accessVersion: bigint;
    constructor(fields: Fields<Sync>);
}
export declare class ServerLeave extends GameEvent {
    serverAddr: string;
    transactorAddr: string;
    constructor(fields: Fields<ServerLeave>);
}
export declare class Leave extends GameEvent {
    playerAddr: string;
    constructor(fields: Fields<Leave>);
}
export declare class GameStart extends GameEvent {
    accessVersion: bigint;
    constructor(fields: Fields<GameStart>);
}
export declare class WaitingTimeout extends GameEvent {
    constructor(_?: any);
}
export declare class DrawRandomItems extends GameEvent {
    sender: string;
    randomId: Id;
    indexes: number[];
    constructor(fields: Fields<DrawRandomItems>);
}
export declare class DrawTimeout extends GameEvent {
    constructor(_: {});
}
export declare class ActionTimeout extends GameEvent {
    playerAddr: string;
    constructor(fields: Fields<ActionTimeout>);
}
export declare class AnswerDecision extends GameEvent {
    sender: string;
    decisionId: Id;
    ciphertext: Uint8Array;
    digest: Uint8Array;
    constructor(fields: Fields<AnswerDecision>);
}
export declare class SecretsReady extends GameEvent {
    constructor(_?: any);
}
export declare class Shutdown extends GameEvent {
    constructor(_?: any);
}
