import { RandomState, RandomSpec } from './random-state';
import { DecisionState } from './decision-state';
import { CiphertextAndDigest, GameEvent, SecretShare } from './events';
import { Effect, Settle } from './effect';
import { GameAccount, PlayerJoin, ServerJoin } from './accounts';
import { Ciphertext, Digest, Id } from './types';
export type NodeStatus = {
    kind: 'pending';
    accessVersion: bigint;
} | {
    kind: 'ready';
} | {
    kind: 'disconnected';
};
export type GameStatus = 'uninit' | 'running' | 'closed';
export interface IPlayer {
    addr: string;
    position: number;
    balance: bigint;
    status: NodeStatus;
}
export interface IServer {
    addr: string;
    status: NodeStatus;
    endpoint: string;
}
export interface DispatchEvent {
    timeout: bigint;
    event: GameEvent;
}
export declare class GameContext {
    gameAddr: string;
    accessVersion: bigint;
    settleVersion: bigint;
    transactorAddr: string;
    status: GameStatus;
    players: IPlayer[];
    servers: IServer[];
    dispatch: DispatchEvent | undefined;
    handlerState: Uint8Array;
    timestamp: bigint;
    allowExit: boolean;
    randomStates: RandomState[];
    decisionStates: DecisionState[];
    settles: Settle[] | undefined;
    constructor(context: GameContext);
    constructor(gameAccount: GameAccount);
    getServerByAddress(addr: string): IServer | undefined;
    getPlayerByAddress(addr: string): IPlayer | undefined;
    dispatchEvent(event: GameEvent, timeout: bigint): void;
    dispatchEventInstantly(event: GameEvent): void;
    waitTimeout(timeout: bigint): void;
    actionTimeout(playerAddr: string, timeout: bigint): void;
    genStartGameEvent(): GameEvent;
    startGame(): void;
    shutdownGame(): void;
    getRandomState(randomId: Id): RandomState;
    getDecisionState(decisionId: Id): DecisionState;
    assign(randomId: Id, playerAddr: string, indexes: number[]): void;
    reveal(randomId: Id, indexes: number[]): void;
    isRandomReady(randomId: Id): boolean;
    isAllRandomReady(): boolean;
    isSecretsReady(): boolean;
    setPlayerStatus(addr: string, status: NodeStatus): void;
    addPlayer(player: PlayerJoin): void;
    addServer(server: ServerJoin): void;
    setAccessVersion(accessVersion: bigint): void;
    setAllowExit(allowExit: boolean): void;
    removePlayer(addr: string): void;
    initRandomState(spec: RandomSpec): Id;
    addSharedSecrets(_addr: string, shares: SecretShare[]): void;
    randomizeAndMask(addr: string, randomId: Id, ciphertexts: Ciphertext[]): void;
    lock(addr: string, randomId: Id, ciphertextsAndTests: CiphertextAndDigest[]): void;
    dispatchRandomizationTimeout(randomId: Id): void;
    settle(settles: Settle[]): void;
    bumpSettleVersion(): void;
    applyAndTakeSettles(): Settle[] | undefined;
    addSettle(settle: Settle): void;
    addRevealedRandom(randomId: Id, revealed: Map<number, string>): void;
    addRevealedAnswer(decisionId: Id, revealed: string): void;
    ask(owner: string): Id;
    answerDecision(id: Id, owner: string, ciphertext: Ciphertext, digest: Digest): void;
    getRevealed(randomId: Id): Map<number, string>;
    applyEffect(effect: Effect): void;
    setNodeReady(accessVersion: bigint): void;
    applyCheckpoint(accessVersion: bigint, settleVersion: bigint): void;
}
