"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Handler_encryptor, _Handler_instance;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Handler = exports.InitAccount = void 0;
const borsh_1 = require("@race-foundation/borsh");
const accounts_1 = require("./accounts");
const events_1 = require("./events");
const effect_1 = require("./effect");
class InitAccount {
    constructor(fields) {
        this.addr = fields.addr;
        this.accessVersion = fields.accessVersion;
        this.settleVersion = fields.settleVersion;
        this.data = fields.data;
        this.players = fields.players;
        this.servers = fields.servers;
    }
    static createFromGameAccount(gameAccount, transactorAccessVersion, transactorSettleVersion) {
        let { addr, players, servers, data } = gameAccount;
        players = players.filter(p => p.accessVersion <= transactorAccessVersion);
        servers = servers.filter(s => s.accessVersion <= transactorAccessVersion);
        return new InitAccount({
            addr,
            data,
            players,
            servers,
            accessVersion: transactorAccessVersion,
            settleVersion: transactorSettleVersion,
        });
    }
    serialize() {
        return (0, borsh_1.serialize)(InitAccount);
    }
    static deserialize(data) {
        return (0, borsh_1.deserialize)(InitAccount, data);
    }
}
__decorate([
    (0, borsh_1.field)('string')
], InitAccount.prototype, "addr", void 0);
__decorate([
    (0, borsh_1.field)((0, borsh_1.array)((0, borsh_1.struct)(accounts_1.PlayerJoin)))
], InitAccount.prototype, "players", void 0);
__decorate([
    (0, borsh_1.field)((0, borsh_1.array)((0, borsh_1.struct)(accounts_1.ServerJoin)))
], InitAccount.prototype, "servers", void 0);
__decorate([
    (0, borsh_1.field)('u8-array')
], InitAccount.prototype, "data", void 0);
__decorate([
    (0, borsh_1.field)('u64')
], InitAccount.prototype, "accessVersion", void 0);
__decorate([
    (0, borsh_1.field)('u64')
], InitAccount.prototype, "settleVersion", void 0);
exports.InitAccount = InitAccount;
class Handler {
    constructor(instance, encryptor) {
        _Handler_encryptor.set(this, void 0);
        _Handler_instance.set(this, void 0);
        __classPrivateFieldSet(this, _Handler_encryptor, encryptor, "f");
        __classPrivateFieldSet(this, _Handler_instance, instance, "f");
    }
    static async initialize(gameBundle, encryptor) {
        const importObject = {
            imports: {
                memory: new WebAssembly.Memory({
                    shared: true,
                    maximum: 100,
                    initial: 100,
                }),
            },
        };
        let initiatedSource;
        if (gameBundle.data.length === 0) {
            console.debug("Initiate handler by streaming:", gameBundle.uri);
            initiatedSource = await WebAssembly.instantiateStreaming(fetch(gameBundle.uri), importObject);
        }
        else {
            initiatedSource = await WebAssembly.instantiate(gameBundle.data, importObject);
        }
        return new Handler(initiatedSource.instance, encryptor);
    }
    async handleEvent(context, event) {
        await this.generalPreHandleEvent(context, event, __classPrivateFieldGet(this, _Handler_encryptor, "f"));
        await this.customHandleEvent(context, event);
        await this.generalPostHandleEvent(context, event);
        context.applyAndTakeSettles();
    }
    async initState(context, initAccount) {
        await this.generalPreInitState(context, initAccount);
        await this.customInitState(context, initAccount);
        await this.generalPostInitState(context, initAccount);
    }
    async generalPreInitState(_context, _initAccount) { }
    async generalPostInitState(_context, _initAccount) { }
    async generalPreHandleEvent(context, event, encryptor) {
        if (event instanceof events_1.ShareSecrets) {
            const { sender, shares } = event;
            context.addSharedSecrets(sender, shares);
            if (context.isSecretsReady()) {
                context.dispatchEventInstantly(new events_1.SecretsReady());
            }
        }
        else if (event instanceof events_1.AnswerDecision) {
            const { decisionId, ciphertext, sender, digest } = event;
            context.answerDecision(decisionId, sender, ciphertext, digest);
        }
        else if (event instanceof events_1.Mask) {
            const { sender, randomId, ciphertexts } = event;
            context.randomizeAndMask(sender, randomId, ciphertexts);
        }
        else if (event instanceof events_1.Lock) {
            const { sender, randomId, ciphertextsAndDigests } = event;
            context.lock(sender, randomId, ciphertextsAndDigests);
        }
        else if (event instanceof events_1.Sync) {
            const { accessVersion, newPlayers, newServers } = event;
            if (accessVersion < context.accessVersion) {
                throw new Error('Event ignored');
            }
            for (const p of newPlayers) {
                context.addPlayer(p);
            }
            for (const s of newServers) {
                context.addServer(s);
            }
            context.accessVersion = accessVersion;
        }
        else if (event instanceof events_1.Leave) {
            const { playerAddr } = event;
            const exist = context.players.find(p => p.addr === playerAddr);
            if (exist === undefined) {
                throw new Error('Invalid player address');
            }
            else {
                context.removePlayer(playerAddr);
            }
        }
        else if (event instanceof events_1.GameStart) {
            const { accessVersion } = event;
            context.status = 'running';
            context.setNodeReady(accessVersion);
        }
        else if (event instanceof events_1.SecretsReady) {
            for (const st of context.randomStates) {
                const options = st.options;
                const revealed = await encryptor.decryptWithSecrets(st.listRevealedCiphertexts(), st.listRevealedSecrets(), options);
                context.addRevealedRandom(st.id, revealed);
            }
        }
    }
    async generalPostHandleEvent(_context, _event) { }
    async customInitState(context, initAccount) {
        const exports = __classPrivateFieldGet(this, _Handler_instance, "f").exports;
        const mem = exports.memory;
        let buf = new Uint8Array(mem.buffer);
        const effect = effect_1.Effect.fromContext(context);
        const effectBytes = (0, borsh_1.serialize)(effect);
        const effectSize = effectBytes.length;
        const initAccountBytes = (0, borsh_1.serialize)(initAccount);
        const initAccountSize = initAccountBytes.length;
        let offset = 1;
        buf.set(effectBytes, offset);
        offset += effectSize;
        buf.set(initAccountBytes, offset);
        const initState = exports.init_state;
        const newEffectSize = initState(effectSize, initAccountSize);
        const data = new Uint8Array(mem.buffer);
        const newEffectBytes = data.slice(1, newEffectSize + 1);
        const newEffect = (0, borsh_1.deserialize)(effect_1.Effect, newEffectBytes);
        if (newEffect.error !== undefined) {
            console.error(newEffect.error);
            throw newEffect.error;
        }
        else {
            context.applyEffect(newEffect);
        }
    }
    async customHandleEvent(context, event) {
        const exports = __classPrivateFieldGet(this, _Handler_instance, "f").exports;
        const mem = exports.memory;
        let buf = new Uint8Array(mem.buffer);
        const effect = effect_1.Effect.fromContext(context);
        const effectBytes = (0, borsh_1.serialize)(effect);
        const effectSize = effectBytes.length;
        const eventBytes = (0, borsh_1.serialize)(event);
        const eventSize = eventBytes.length;
        let offset = 1;
        buf.set(effectBytes, offset);
        offset += effectSize;
        buf.set(eventBytes, offset);
        const handleEvent = exports.handle_event;
        const newEffectSize = handleEvent(effectSize, eventSize);
        const data = new Uint8Array(mem.buffer);
        const newEffectBytes = data.slice(1, newEffectSize + 1);
        const newEffect = (0, borsh_1.deserialize)(effect_1.Effect, newEffectBytes);
        if (newEffect.error !== undefined) {
            throw newEffect.error;
        }
        else {
            context.applyEffect(newEffect);
        }
    }
}
exports.Handler = Handler;
_Handler_encryptor = new WeakMap(), _Handler_instance = new WeakMap();
//# sourceMappingURL=handler.js.map