var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Encryptor_privateKey, _Encryptor_publicKeys;
import { SdkError } from './error';
import { field } from '@race-foundation/borsh';
import { base64ToArrayBuffer, arrayBufferToBase64 } from './utils';
import { Chacha20 } from 'ts-chacha20';
let subtle = crypto.subtle;
export const aesContentIv = Uint8Array.of(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
export const chacha20Nonce = Uint8Array.of(1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
export const aesDigestIv = Uint8Array.of(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1);
const textDecoder = new TextDecoder('utf8');
const publicExponent = Uint8Array.of(1, 0, 1);
export class PublicKeyRaws {
    constructor(fields) {
        this.rsa = fields.rsa;
        this.ec = fields.ec;
    }
}
__decorate([
    field('string')
], PublicKeyRaws.prototype, "rsa", void 0);
__decorate([
    field('string')
], PublicKeyRaws.prototype, "ec", void 0);
const RSA_PARAMS = {
    name: 'RSA-OAEP',
    hash: 'SHA-256',
};
const EC_PARAMS = {
    name: 'ECDSA',
    namedCurve: 'P-256',
};
export async function exportRsaPublicKey(publicKey) {
    return arrayBufferToBase64(await subtle.exportKey('spki', publicKey));
}
export async function exportEcPublicKey(publicKey) {
    return arrayBufferToBase64(await subtle.exportKey('spki', publicKey));
}
export async function exportAes(key) {
    return new Uint8Array(await subtle.exportKey('raw', key));
}
export async function exportRsa(keypair) {
    let privkey = await subtle.exportKey('pkcs8', keypair.privateKey);
    return [arrayBufferToBase64(privkey), await exportRsaPublicKey(keypair.publicKey)];
}
export async function exportEc(keypair) {
    let privkey = await subtle.exportKey('pkcs8', keypair.privateKey);
    return [arrayBufferToBase64(privkey), await exportEcPublicKey(keypair.publicKey)];
}
export async function encryptRsa(publicKey, plaintext) {
    return new Uint8Array(await subtle.encrypt('RSA-OAEP', publicKey, plaintext));
}
export async function decryptRsa(privateKey, ciphertext) {
    return new Uint8Array(await subtle.decrypt('RSA-OAEP', privateKey, ciphertext));
}
export async function signEc(privateKey, message) {
    return new Uint8Array(await subtle.sign({ name: 'ECDSA', hash: { name: 'SHA-256' } }, privateKey, message));
}
export async function verifyEc(publicKey, signature, message) {
    return await subtle.verify({ name: 'ECDSA', hash: { name: 'SHA-256' } }, publicKey, signature, message);
}
export function encryptChacha20(key, text, nonce) {
    return new Chacha20(key, nonce).encrypt(text);
}
export function decryptChacha20(key, text, nonce) {
    return new Chacha20(key, nonce).decrypt(text);
}
export async function encryptAes(key, text, iv) {
    return new Uint8Array(await subtle.encrypt({
        name: 'AES-CTR',
        counter: iv,
        length: 64,
    }, key, text));
}
export async function decryptAes(key, text, iv) {
    return new Uint8Array(await subtle.decrypt({
        name: 'AES-CTR',
        counter: iv,
        length: 64,
    }, key, text));
}
export async function importAes(rawKey) {
    return await subtle.importKey('raw', rawKey, { name: 'AES-CTR' }, true, ['encrypt', 'decrypt']);
}
export async function importRsa([privateKeyStr, publicKeyStr]) {
    const privateBuf = base64ToArrayBuffer(privateKeyStr);
    const privateKey = await subtle.importKey('pkcs8', privateBuf, RSA_PARAMS, true, ['decrypt']);
    const publicKey = await importRsaPublicKey(publicKeyStr);
    return { publicKey, privateKey };
}
export async function importEc([privateKeyStr, publicKeyStr]) {
    const privateBuf = base64ToArrayBuffer(privateKeyStr);
    const privateKey = await subtle.importKey('pkcs8', privateBuf, EC_PARAMS, true, ['sign']);
    const publicKey = await importEcPublicKey(publicKeyStr);
    return { publicKey, privateKey };
}
export async function importRsaPublicKey(publicKeyStr) {
    const publicBuf = base64ToArrayBuffer(publicKeyStr);
    const publicKey = await subtle.importKey('spki', publicBuf, RSA_PARAMS, true, ['encrypt']);
    return publicKey;
}
export async function importEcPublicKey(publicKeyStr) {
    const publicBuf = base64ToArrayBuffer(publicKeyStr);
    const publicKey = await subtle.importKey('spki', publicBuf, EC_PARAMS, true, ['verify']);
    return publicKey;
}
export async function generateEcKeypair() {
    return await subtle.generateKey(EC_PARAMS, true, ['verify', 'sign']);
}
export async function generateRsaKeypair() {
    return await subtle.generateKey({
        name: 'RSA-OAEP',
        modulusLength: 1024,
        publicExponent: publicExponent,
        hash: 'SHA-256',
    }, true, ['encrypt', 'decrypt']);
}
export function generateChacha20() {
    const arr = new Uint8Array(32);
    crypto.getRandomValues(arr);
    return arr;
}
export async function generateAes() {
    const k = await subtle.generateKey({
        name: 'AES-CTR',
        length: 128,
    }, true, ['encrypt', 'decrypt']);
    return k;
}
export class Signature {
    constructor(fields) {
        this.signer = fields.signer;
        this.timestamp = fields.timestamp;
        this.signature = fields.signature;
    }
}
__decorate([
    field('string')
], Signature.prototype, "signer", void 0);
__decorate([
    field('u64')
], Signature.prototype, "timestamp", void 0);
__decorate([
    field('u8-array')
], Signature.prototype, "signature", void 0);
class NodePrivateKey {
    constructor(rsa, ec) {
        this.rsa = rsa;
        this.ec = ec;
    }
    static async initialize(keys) {
        let rsa, ec;
        if (keys?.rsa === undefined) {
            rsa = await generateRsaKeypair();
        }
        else {
            rsa = keys.rsa;
        }
        if (keys?.ec === undefined) {
            ec = await generateEcKeypair();
        }
        else {
            ec = keys.ec;
        }
        return new NodePrivateKey(rsa, ec);
    }
}
class NodePublicKey {
    constructor(rsa, ec) {
        this.rsa = rsa;
        this.ec = ec;
    }
}
export class Encryptor {
    constructor(priv) {
        _Encryptor_privateKey.set(this, void 0);
        _Encryptor_publicKeys.set(this, void 0);
        __classPrivateFieldSet(this, _Encryptor_privateKey, priv, "f");
        __classPrivateFieldSet(this, _Encryptor_publicKeys, new Map(), "f");
    }
    async decryptRsa(text) {
        return await decryptRsa(__classPrivateFieldGet(this, _Encryptor_privateKey, "f").rsa.privateKey, text);
    }
    async decryptAes(secret, text) {
        const key = await importAes(secret);
        console.log(aesContentIv);
        return await decryptAes(key, text, aesContentIv);
    }
    async decryptAesMulti(secrets, text) {
        for (const secret of secrets) {
            text = await this.decryptAes(secret, text);
        }
        return text;
    }
    encryptChacha20(secret, text) {
        return encryptChacha20(secret, text, chacha20Nonce);
    }
    decryptChacha20(secret, text) {
        return decryptChacha20(secret, text, chacha20Nonce);
    }
    decryptChacha20Multi(secrets, text) {
        for (const secret of secrets) {
            text = this.decryptChacha20(secret, text);
        }
        return text;
    }
    async signRaw(message) {
        return await signEc(__classPrivateFieldGet(this, _Encryptor_privateKey, "f").ec.privateKey, message);
    }
    makeSignMessage(message, timestamp) {
        const timestampView = new DataView(new ArrayBuffer(8));
        timestampView.setBigUint64(0, timestamp, true);
        const buf = new Uint8Array(message.length + 8);
        buf.set(message);
        buf.set(new Uint8Array(timestampView.buffer), message.length);
        return buf;
    }
    async sign(message, signer) {
        const timestamp = BigInt(new Date().getTime());
        const buf = this.makeSignMessage(message, timestamp);
        const signature = await this.signRaw(buf);
        return new Signature({
            timestamp,
            signer,
            signature,
        });
    }
    async verify(message, signature) {
        const timestamp = signature.timestamp;
        const ecPublicKey = __classPrivateFieldGet(this, _Encryptor_publicKeys, "f").get(signature.signer)?.ec;
        if (ecPublicKey === undefined) {
            throw new Error("Can't verify message, ECDSA key is missing");
        }
        const buf = this.makeSignMessage(message, timestamp);
        return await verifyEc(ecPublicKey, signature.signature, buf);
    }
    static async create() {
        const rsaKeypair = await generateRsaKeypair();
        const ecKeypair = await generateEcKeypair();
        return new Encryptor(new NodePrivateKey(rsaKeypair, ecKeypair));
    }
    async exportRsaKeys() {
        return await exportRsa(__classPrivateFieldGet(this, _Encryptor_privateKey, "f").rsa);
    }
    async exportEcKeys() {
        return await exportEc(__classPrivateFieldGet(this, _Encryptor_privateKey, "f").ec);
    }
    async addPublicKey(addr, { rsa, ec }) {
        const rsa_ = await importRsaPublicKey(rsa);
        const ec_ = await importEcPublicKey(ec);
        __classPrivateFieldGet(this, _Encryptor_publicKeys, "f").set(addr, new NodePublicKey(rsa_, ec_));
    }
    async exportPublicKey(addr) {
        let rsa, ec;
        if (addr === undefined) {
            rsa = __classPrivateFieldGet(this, _Encryptor_privateKey, "f").rsa.publicKey;
            ec = __classPrivateFieldGet(this, _Encryptor_privateKey, "f").ec.publicKey;
        }
        else {
            const publicKeys = __classPrivateFieldGet(this, _Encryptor_publicKeys, "f").get(addr);
            if (publicKeys === undefined) {
                throw SdkError.publicKeyNotFound(addr);
            }
            rsa = publicKeys.rsa;
            ec = publicKeys.ec;
        }
        return new PublicKeyRaws({ rsa: await exportRsaPublicKey(rsa), ec: await exportEcPublicKey(ec) });
    }
    async decryptWithSecrets(ciphertextMap, secretMap, validOptions) {
        console.log("Ciphertext Map:", ciphertextMap);
        console.log("Secret Map:", secretMap);
        const res = new Map();
        for (const [idx, ciphertext] of ciphertextMap) {
            const secrets = secretMap.get(idx);
            if (secrets === undefined) {
                throw new Error('Missing secrets');
            }
            else {
                const decrypted = await this.decryptChacha20Multi(secrets, ciphertext);
                const decryptedValue = textDecoder.decode(decrypted);
                if (validOptions.find(s => s === decryptedValue) === undefined) {
                    console.log("Options:", validOptions);
                    console.log(decryptedValue);
                    throw new Error('Invalid result: [' + decryptedValue + "], options:" + validOptions.join(","));
                }
                res.set(idx, decryptedValue);
            }
        }
        return res;
    }
}
_Encryptor_privateKey = new WeakMap(), _Encryptor_publicKeys = new WeakMap();
//# sourceMappingURL=encryptor.js.map